const _ = require('lodash');
const uts46 = require('idna-uts46-hx');

const EthUtils = require('web3-utils');

Object.assign(EthUtils.unitMap, {
  raw: EthUtils.unitMap.wei,
  eth: EthUtils.unitMap.ether,
  token: EthUtils.unitMap.ether,
  tokens: EthUtils.unitMap.ether
});

_.mixin(EthUtils);

_.mixin({
  exists(val) { return (typeof val !== 'undefined' && val !== null); },

  logs(tx, ev) {
    return tx.logs.filter(log => log.event === ev).map(log => log.args);
  },

  toHex(val, len) { return '0x' + EthUtils.toBN(val).toString(16, len); },

  toBytes32(val) {
    if (Array.isArray(val)) {
      return val.map((v) => { return _.toBytes32(v); });
    } else {
      return this.toHex(val, 32);
    }
  },

  namehash(name='') {
    return name.split('.').reduceRight((node, label) => {
      if (label) { return _.soliditySha3(node, _.soliditySha3({t: 'string', v: label})); }
      else { return node; }
    }, _.toHex(0, 64));
  },
  ethUnitMap() { return EthUtils.unitMap; },
  convertTo(val, to='bn', label=true) {
    to = _.toLower(to);
    if (_.isString(val)) {
      val =_.toWei.apply(this, (val.match(/^\s*(\d+\.?\d*)\s*(\w*)/) || [,'0']).slice(1,3));
    }
    if (to === 'bn') { return _.toBN(val); }
    val = _.fromWei(val.toString(), to);
    return label ? val + ' ' + to : val;
  }
}, { chain: false });

module.exports = _;

