const levelup = require('levelup');
const Cache = require('../../lib/cache.js');

describe('cache', () => {
  beforeEach( async () => {
    this.cache = new Cache();
  });

  describe('#constructor', () => {
    it('should set defaults', async () => {
      this.cache.max.should.equal(500);
      this.cache.ttl.should.equal(0);
    });
  });

  describe('#set', () => {
    it('should add a value', async () => {
      await this.cache.set('hello', 'world', 10);
      this.cache.lru.size.should.equal(1);
    });
  });

  describe('#get', () => {
    it('should get a value', async () => {
      await this.cache.set('hello', 'world', 10);
      let result = await this.cache.get('hello');
      result.should.equal('world');
    });

    it('should return undefined for a uncached key', async () => {
      let result = await this.cache.get('hell');
      expect(result).to.equal(undefined);
    });
  });

  describe('#memoize', () => {
    it('should return a function', async () => {
      let func = (x) => { return x * 2; };
      let result = this.cache.memoize(func);
      expect(typeof result).to.equal('function');
    });

    it('should memoized function should return expected results', async () => {
      let func = (x) => { return x * 2; };
      let memo = this.cache.memoize(func);
      let result = await memo(2);
      result.should.equal(4);
    });

    it('should set the correct this', async () => {
      let func = function (x) { return x * this.multi; };
      let obj = { multi: 4, memo: this.cache.memoize(func) };
      let result = await obj.memo(2);
      result.should.equal(8);
    });

    it('should get key from resolver', async () => {
      let func = function (x) { return x * this.multi; };
      let resolver = function (x) { return x + this.multi; };
      let obj = { multi: 4, memo: this.cache.memoize(func, resolver, 10) };
      let result = await obj.memo(1);
      expect(this.cache.lru._cache.has(5)).to.equal(true);
    });
  });

});
