const IPFS = require('ipfs-http-client');

const API = require('../../lib');

describe('domain', () => {
  [_, owner, investor, benefactor, hacker, other] = accounts;

  before( async () => {
    this.name = 'DwNS Domain';
    this.symbol = 'DWNS';

    this.contracts = await initDwNS(this.name, this.symbol);
    this.app = this.contracts.app;

    this.options = {
      ethereum: web3.currentProvider,
      ipfs: new IPFS(),
      from: _,
      address: this.app.address
    };

    this.api = await API.setup(this.options);

    await this.api.registry.register(_, nh(''), 'domain');
    await this.api.registry.register(_, nh('domain'), 'one');
    await this.api.registry.register(_, nh('domain'), 'two');
  });

  describe('#createToken', () => {
    it('should create a token', async () => {
      let domain = await this.api.resolve('one.domain');
      let addr = await domain.createToken('Test', 'TST');
      let token = await domain.details('token');
      token.should.equal(addr);
    });
  });

  describe('#token', () => {
    it('should return a RegistrarToken instance', async () => {
      let domain = await this.api.resolve('one.domain');
      let addr = await domain.details('token');
      let result = await domain.token();
      result.address.should.equal(addr);
    });
  });

  describe('#record', () => {
    it('should return the default record data if no record has been set', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.record('/name');
      result.should.equal('Empty Record');
    });

    it('should return an ipfs link when passed data', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.record({hello: 'world'});
      result.should.equal('zdpuAtX7ZibcWdSKQwiDCkPjWwRvtcKCPku9H7LhgA4qJW4Wk');
    });

    it('should return the record data if no data passed', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.record('/hello');
      result.should.equal('world');
    });
  });

  describe('#owner', () => {
    it('should return owner', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.owner();
      result.should.equal(_);
    });
  });

  describe('#subdomains', () => {
    it('should return subdomains', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.subdomains(1, 2);
      result.length.should.equal(1);
      result[0].id.should.eql(nh('two.domain'));
    });
  });

  describe('#snapshot', () => {
    it('should return snapshot', async () => {
      let domain = await this.api.resolve('domain');
      let result = await domain.snapshot();
      expect(result.tld).to.equal(true);
    });
  });

  describe('#meta', () => {
    it('should return meta data', async () => {
      let domain = await this.api.resolve('one.domain');
      let result = await domain.meta();
      result.attributes[0].value.should.equal('one.domain');
    });
  });
});
