const IPFS = require('ipfs-http-client');

const API = require('../../lib');
const Domain = require('../../lib/domain.js');

describe('DwNS', () => {
  [_, owner, investor, benefactor, hacker, other] = accounts;

  before( async () => {
    this.name = 'DwNS Domain';
    this.symbol = 'DWNS';

    this.contracts = await initDwNS(this.name, this.symbol);
    this.app = this.contracts.app;

    this.options = {
      ethereum: web3.currentProvider,
      ipfs: new IPFS(),
      from: _,
      address: this.app.address
    };

    this.api = await API.setup(this.options);

    await this.api.registry.register(_, nh(''), 'app');
    await this.api.registry.register(owner, nh('app'), 'one');
    await this.api.registry.register(owner, nh('app'), 'two');
  });

  describe('#resolve', () => {
    it('should return a domain instance', async () => {
      let result = await this.api.resolve('app');
      result.should.be.an.instanceof(Domain);
    });
  });

  describe('#domainsOwnedBy', () => {
    it('should return a domains of owner', async () => {
      let result = await this.api.domainsOwnedBy(owner);

      result.length.should.equal(2);
      result[0].id.should.eql(nh('one.app'));
      result[1].id.should.eql(nh('two.app'));
    });
  });

  describe('#domains', () => {
    it('should return a domains of owner', async () => {
      let result = await this.api.domains();

      result.length.should.equal(2);
      result[0].id.should.eql(nh(''));
      result[1].id.should.eql(nh('app'));
    });
  });
});

