const BigNumber = web3.BigNumber;
const encodeCall = require('zos-lib/lib/helpers/encodeCall').default;
const Helpers = require('../lib/helpers.js');

const nh = Helpers.namehash;
const bytes32 = Helpers.toBytes32;

require('chai')
  .use(require('chai-bignumber')(BigNumber))
  .use(require('chai-as-promised'))
  .should();

const Foundry = artifacts.require('FoundryV0');
const DirectoryToken = artifacts.require('DirectoryTokenV0');
const Directory = artifacts.require('DirectoryV0');

contract('Directory', ([_, owner, investor, benefactor, hacker]) => {
  beforeEach( async () => {
    this.name = "DwNS Token";
    this.symbol = "DWNS";

    this.foundry = await Foundry.new({ from: _ });
    this.directoryToken = await DirectoryToken.new({ from: _ });

    await this.foundry.setMold(
      'DirectoryToken',
      this.directoryToken.address,
      { from: _ }
    );

    this.directory = await Directory.new({ from: _ });
    const callData = encodeCall(
      'initialize',
        ['address', 'address', 'address', 'string', 'string'],
        [_, this.foundry.address, _, this.name, this.symbol]
    );
    await this.directory.sendTransaction({data: callData, from: _});
  });

  describe('properties', () => {
    it('should have a name', async () => {
      const result = await this.directory.name();
      result.should.equal(this.name);
    });
  });

  describe('getters', () => {
    it('should return zones of owner', async () => {
      await this.directory.mint(investor, 0, 'test');
      const result = await this.directory.zonesOfOwner(investor);
      result.length.should.equal(1);
      bytes32(result)[0].should.equal(nh('test'));
    });

    it('should return zone details', async () => {
      await this.directory.mint(investor, 0, 'test');
      const result = await this.directory.zones(nh('test'));
        console.log(result);
      result.length.should.equal(6);
      result[0].should.be.bignumber.equal(0);
      result[1].should.be.equal('test');
      result[2].should.be.equal('test');
    });

    // it('should return zone children', async () => {
    //   await this.directory.mint(investor, 0, 'test1');
    //   await this.directory.mint(investor, 0, 'test2');
    //   const result = await this.directory.subzones(0);
    //   result.length.should.equal(2);
    //   result[0].should.be.bignumber.equal(1);
    //   result[1].should.be.bignumber.equal(2);
    // });
  });

  // describe('minting', () => {
  //   describe('valid', () => {
  //     it('should mint a tlz', async () => {
  //       const receipt = await this.directory.mint(investor, 'test');
  //       const zone = await this.directory.zones.call(1);
  //       zone[2].should.equal('test');
  //     });
  //   });

  //   describe('invalid', () => {
  //     it('should reject for an invalid parent fqn', async () => {
  //       await this.directory.mint(benefactor, '1.test').should.be.rejectedWith('revert');
  //     });

  //     it('should reject for invalid characters', async () => {
  //       const callData = encodeCall('mint', ['address', 'uint', 'string'], [benefactor, 0, '1.test']);
  //       // await this.directory.sendTransaction({data: callData, from: _}).should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1 test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1,test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1@test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1:test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1#test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1"test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, "1'test").should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1/test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\\test').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\ntest').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\rtest').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\ttest').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\btest').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\ftest').should.be.rejectedWith('revert');
  //       await this.directory.mint(benefactor, '1\vtest').should.be.rejectedWith('revert');
  //     });
  //   });
  // });
});
