const Web3 = require('web3');
const Contract = require('truffle-contract');

global.truffleConfig = require('../../truffle-config.js').networks.local;

global.providerUrl = 'http://' + truffleConfig.host + ':' + truffleConfig.port;
global.provider = new Web3.providers.HttpProvider(providerUrl);
global.web3 = new Web3(provider);

global.accounts = web3.eth.accounts;

global.artifacts = {
  require(name) {
    let schema = require('../../build/contracts/' + name + '.json');
    let contract = Contract(schema);
    contract.defaults({gas: truffleConfig.gas, from: accounts[0]});
    contract.setNetwork(truffleConfig.network_id);
    contract.setProvider(web3.currentProvider);
    return contract;
  }
};

const App = artifacts.require('App');
const Registry = artifacts.require('RegistryV0');
const OwnershipToken = artifacts.require('OwnershipTokenV0');
const RegistrarToken = artifacts.require('RegistrarTokenV0');

global.initDwNS = async (name, symbol) => {
  let app = await App.new();
  await app.addPackage('Registry', Registry.address);
  await app.addPackage('OwnershipToken', OwnershipToken.address);
  await app.addPackage('RegistrarToken', RegistrarToken.address);

  let tx = await app.create('Registry');
  let registry = Registry.at(tx.logs[0].args.addr);

  await registry.initialize(app.address, name, symbol);
  let tokenProxy = await registry.ownershipToken();

  await app.set('registry', registry.address);
  await app.set('token', tokenProxy);

  return {app, registry};
};

const BigNumber = web3.BigNumber;


const Chai = require('chai');

Chai
  .use(require('chai-bignumber')(web3.BigNumber))
  .use(require('chai-as-promised'))
  .should();

global.expect = Chai.expect;

global.EmptyAddr = '0x0000000000000000000000000000000000000000';
global.encodeCall = require('zos-lib/lib/helpers/encodeCall').default;

global.Helpers = require('../../lib/helpers.js');
global.nh = Helpers.namehash;
global.logs = Helpers.logs;
